"""
PI-LLM-Server - 统一 LLM 服务网关

集成 Embedding、ASR、Reranker、MinerU 四个子服务，
提供统一的 API 入口、请求队列、认证管理、健康监控等功能。
"""

from importlib.metadata import version

__version__ = version("pi-llm-server")
__author__ = "PI-Lab Team"

from .config import ConfigManager, Config, ServiceConfig
from .auth import AuthManager
from .queue_manager import QueueManager, ServiceQueueConfig

__all__ = [
    "ConfigManager",
    "Config",
    "ServiceConfig",
    "AuthManager",
    "QueueManager",
    "ServiceQueueConfig",
]
